@@ -5,27 +5,32 @@ from collections.abc import Iterator
 from pydantic import validate_call
 from requests.exceptions import RetryError
 
-from albert.collections.base import BaseCollection, OrderBy
-from albert.exceptions import AlbertHTTPError
-from albert.resources.identifiers import (
+from albert.collections.base import BaseCollection
+from albert.core.logging import logger
+from albert.core.pagination import AlbertPaginator
+from albert.core.session import AlbertSession
+from albert.core.shared.enums import OrderBy, PaginationMode
+from albert.core.shared.identifiers import (
     BlockId,
     DataTemplateId,
     TaskId,
     WorkflowId,
 )
+from albert.core.shared.models.base import EntityLink, EntityLinkWithName
+from albert.core.shared.models.patch import PatchOperation
+from albert.exceptions import AlbertHTTPError
 from albert.resources.tasks import (
     BaseTask,
+    BatchTask,
+    GeneralTask,
     HistoryEntity,
     PropertyTask,
     TaskAdapter,
     TaskCategory,
     TaskHistory,
     TaskPatchPayload,
+    TaskSearchItem,
 )
-from albert.session import AlbertSession
-from albert.utils.logging import logger
-from albert.utils.pagination import AlbertPaginator, PaginationMode
-from albert.utils.patches import PatchDatum, PatchOperation, PatchPayload
 
 
 class TaskCollection(BaseCollection):
@@ -38,7 +43,6 @@ class TaskCollection(BaseCollection):
         "priority",
         "state",
         "tags",
-        "assigned_to",
         "due_date",
     }
 
@@ -53,12 +57,12 @@ class TaskCollection(BaseCollection):
         super().__init__(session=session)
         self.base_path = f"/api/{TaskCollection._api_version}/tasks"
 
-    def create(self, *, task: BaseTask) -> BaseTask:
+    def create(self, *, task: PropertyTask | GeneralTask | BatchTask) -> BaseTask:
         """Create a new task. Tasks can be of different types, such as PropertyTask, and are created using the provided task object.
 
         Parameters
         ----------
-        task : BaseTask
+        task : PropertyTask | GeneralTask | BatchTask
             The task object to create.
 
         Returns
@@ -109,7 +113,6 @@ class TaskCollection(BaseCollection):
             }
         ]
         self.session.patch(url=url, json=payload)
-        return None
 
     @validate_call
     def update_block_workflow(
@@ -170,7 +173,6 @@ class TaskCollection(BaseCollection):
             }
         ]
         self.session.patch(url=url, json=patch)
-        return None
 
     @validate_call
     def remove_block(self, *, task_id: TaskId, block_id: BlockId) -> None:
@@ -201,7 +203,6 @@ class TaskCollection(BaseCollection):
             }
         ]
         self.session.patch(url=url, json=payload)
-        return None
 
     @validate_call
     def delete(self, *, id: TaskId) -> None:
@@ -233,12 +234,11 @@ class TaskCollection(BaseCollection):
         response = self.session.get(url)
         return TaskAdapter.validate_python(response.json())
 
-    def list(
+    @validate_call
+    def search(
         self,
         *,
-        order: OrderBy = OrderBy.DESCENDING,
         text: str | None = None,
-        sort_by: str | None = None,
         tags: list[str] | None = None,
         task_id: list[str] | None = None,
         linked_task: list[str] | None = None,
@@ -252,67 +252,64 @@ class TaskCollection(BaseCollection):
         parameter_group: list[str] | None = None,
         created_by: list[str] | None = None,
         project_id: str | None = None,
-        limit: int = 100,
+        order_by: OrderBy = OrderBy.DESCENDING,
+        sort_by: str | None = None,
+        max_items: int | None = None,
         offset: int = 0,
-    ) -> Iterator[BaseTask]:
-        """Search for tasks matching the given criteria.
+    ) -> Iterator[TaskSearchItem]:
+        """
+        Search for Task matching the provided criteria.
+
+        ⚠️ This method returns partial (unhydrated) entities to optimize performance.
+        To retrieve fully detailed entities, use :meth:`get_all` instead.
 
         Parameters
         ----------
-        order : OrderBy, optional
-            The order in which to return results, by default OrderBy.DESCENDING
-        text : str | None, optional
-            The text to search for, by default None
-        sort_by : str | None, optional
-            The attribute to sort by, by default None
-        tags : list[str] | None, optional
-            The tags to search for, by default None
-        task_id : list[str] | None, optional
-            The related task IDs to search for, by default None
-        linked_task : list[str] | None, optional
-            The Linked Task IDs to search for, by default None
-        category : TaskCategory | None, optional
-            The category of the task to search for, by default None
-        albert_id : list[str] | None, optional
-            The Albert IDs to search for, by default None
-        data_template : list[str] | None, optional
-            The data template IDs to search for, by default None
-        assigned_to : list[str] | None, optional
-            The User IDs to search for, by default None
-        location : list[str] | None, optional
-            The Locations names to search for, by default None
-        priority : list[str] | None, optional
-            The Priority levels to search for, by default None
-        status : list[str] | None, optional
-            The Task Statuses to search for, by default None
-        parameter_group : list[str] | None, optional
-            The related Parameter Group IDs to search for, by default None
-        created_by : list[str] | None, optional
-            The User IDs of the task creators to search for, by default None
-        project_id : str | None, optional
-            The Project ID to search for, by default None
+        text : str, optional
+            Text search across multiple task fields.
+        tags : list[str], optional
+            Filter by tags associated with tasks.
+        task_id : list[str], optional
+            Specific task IDs to search for.
+        linked_task : list[str], optional
+            Task IDs linked to the ones being searched.
+        category : TaskCategory, optional
+            Task category filter (e.g., Experiment, Analysis).
+        albert_id : list[str], optional
+            Albert-specific task identifiers.
+        data_template : list[str], optional
+            Data template IDs associated with tasks.
+        assigned_to : list[str], optional
+            User IDs assigned to the tasks.
+        location : list[str], optional
+            Locations where tasks are carried out.
+        priority : list[str], optional
+            Priority levels for filtering tasks.
+        status : list[str], optional
+            Task status values (e.g., Open, Done).
+        parameter_group : list[str], optional
+            Parameter Group IDs associated with tasks.
+        created_by : list[str], optional
+            User IDs who created the tasks.
+        project_id : str, optional
+            ID of the parent project for filtering tasks.
+        order_by : OrderBy, optional
+            The order in which to return results (asc or desc), default DESCENDING.
+        sort_by : str, optional
+            Attribute to sort tasks by (e.g., createdAt, name).
+        max_items : int, optional
+            Maximum number of items to return in total. If None, fetches all available items.
+        offset : int, optional
+            Number of results to skip for pagination, default 0.
 
-        Yields
-        ------
-        Iterator[BaseTask]
-            An iterator of matching Task objects.
+        Returns
+        -------
+        Iterator[TaskSearchItem]
+            An iterator of matching, lightweight TaskSearchItem entities.
         """
-
-        def deserialize(items: list[dict]) -> Iterator[BaseTask]:
-            for item in items:
-                id = item["albertId"]
-                try:
-                    yield self.get_by_id(id=id)
-                except (
-                    AlbertHTTPError,
-                    RetryError,
-                ) as e:  # some legacy poorly formed Tasks raise 500s. The allowance on Retry error to also ignore these.
-                    logger.warning(f"Error fetching task '{id}': {e}")
-
         params = {
-            "limit": limit,
             "offset": offset,
-            "order": OrderBy(order).value if order else None,
+            "order": order_by.value,
             "text": text,
             "sortBy": sort_by,
             "tags": tags,
@@ -334,10 +331,114 @@ class TaskCollection(BaseCollection):
             mode=PaginationMode.OFFSET,
             path=f"{self.base_path}/search",
             session=self.session,
-            deserialize=deserialize,
             params=params,
+            max_items=max_items,
+            deserialize=lambda items: [
+                TaskSearchItem(**item)._bind_collection(self) for item in items
+            ],
         )
 
+    def get_all(
+        self,
+        *,
+        text: str | None = None,
+        tags: list[str] | None = None,
+        task_id: list[str] | None = None,
+        linked_task: list[str] | None = None,
+        category: TaskCategory | None = None,
+        albert_id: list[str] | None = None,
+        data_template: list[str] | None = None,
+        assigned_to: list[str] | None = None,
+        location: list[str] | None = None,
+        priority: list[str] | None = None,
+        status: list[str] | None = None,
+        parameter_group: list[str] | None = None,
+        created_by: list[str] | None = None,
+        project_id: str | None = None,
+        order_by: OrderBy = OrderBy.DESCENDING,
+        sort_by: str | None = None,
+        max_items: int | None = None,
+        offset: int = 0,
+    ) -> Iterator[BaseTask]:
+        """
+        Retrieve fully hydrated Task entities with optional filters.
+
+        This method returns complete entity data using `get_by_id`.
+        Use :meth:`search` for faster retrieval when you only need lightweight, partial (unhydrated) entities.
+
+        Parameters
+        ----------
+        text : str, optional
+            Text search across multiple task fields.
+        tags : list[str], optional
+            Filter by tags associated with tasks.
+        task_id : list[str], optional
+            Specific task IDs to search for.
+        linked_task : list[str], optional
+            Task IDs linked to the ones being searched.
+        category : TaskCategory, optional
+            Task category filter (e.g., Experiment, Analysis).
+        albert_id : list[str], optional
+            Albert-specific task identifiers.
+        data_template : list[str], optional
+            Data template IDs associated with tasks.
+        assigned_to : list[str], optional
+            User IDs assigned to the tasks.
+        location : list[str], optional
+            Locations where tasks are carried out.
+        priority : list[str], optional
+            Priority levels for filtering tasks.
+        status : list[str], optional
+            Task status values (e.g., Open, Done).
+        parameter_group : list[str], optional
+            Parameter Group IDs associated with tasks.
+        created_by : list[str], optional
+            User IDs who created the tasks.
+        project_id : str, optional
+            ID of the parent project for filtering tasks.
+        order_by : OrderBy, optional
+            The order in which to return results (asc or desc), default DESCENDING.
+        sort_by : str, optional
+            Attribute to sort tasks by (e.g., createdAt, name).
+        max_items : int, optional
+            Maximum number of items to return in total. If None, fetches all available items.
+        offset : int, optional
+            Number of results to skip for pagination, default 0.
+
+        Yields
+        ------
+        Iterator[BaseTask]
+            A stream of fully hydrated Task entities (PropertyTask, BatchTask, or GeneralTask).
+        """
+        for task in self.search(
+            text=text,
+            tags=tags,
+            task_id=task_id,
+            linked_task=linked_task,
+            category=category,
+            albert_id=albert_id,
+            data_template=data_template,
+            assigned_to=assigned_to,
+            location=location,
+            priority=priority,
+            status=status,
+            parameter_group=parameter_group,
+            created_by=created_by,
+            project_id=project_id,
+            order_by=order_by,
+            sort_by=sort_by,
+            max_items=max_items,
+            offset=offset,
+        ):
+            task_id = getattr(task, "id", None)
+            if not task_id:
+                continue
+
+            try:
+                yield self.get_by_id(id=task_id)
+            except (AlbertHTTPError, RetryError) as e:
+                logger.warning(f"Error fetching task '{task_id}': {e}")
+
     def _is_metadata_item_list(
         self,
         *,
@@ -362,100 +463,119 @@ class TaskCollection(BaseCollection):
 
         return isinstance(existing, list) or isinstance(updated, list)
 
-    def _generate_patch_payload(
+    def _generate_task_patch_payload(
         self,
         *,
         existing: BaseTask,
         updated: BaseTask,
-    ) -> tuple[PatchPayload, dict[str, list[str]]]:
+    ) -> TaskPatchPayload:
         """Generate patch payload and capture metadata list updates."""
 
         base_payload = super()._generate_patch_payload(
             existing=existing,
             updated=updated,
+            generate_metadata_diff=True,
         )
-
-        new_data: list[PatchDatum] = []
-        list_metadata_updates: dict[str, list[str]] = {}
-
-        for datum in base_payload.data:
-            if self._is_metadata_item_list(
-                existing_object=existing,
-                updated_object=updated,
-                metadata_field=datum.attribute,
-            ):
-                key = datum.attribute.split(".", 1)[1]
-                updated_list = updated.metadata.get(key) or []
-                list_values: list[str] = [
-                    item.id if hasattr(item, "id") else item for item in updated_list
-                ]
-
-                list_metadata_updates[datum.attribute] = list_values
-                continue
-
-            new_data.append(
-                PatchDatum(
-                    operation=datum.operation,
-                    attribute=datum.attribute,
-                    new_value=datum.new_value,
-                    old_value=datum.old_value,
-                )
-            )
-
-        return TaskPatchPayload(data=new_data, id=existing.id), list_metadata_updates
+        return TaskPatchPayload(data=base_payload.data, id=existing.id)
 
     def _generate_adv_patch_payload(
         self, *, updated: BaseTask, existing: BaseTask
-    ) -> tuple[dict, dict[str, list[str]]]:
+    ) -> TaskPatchPayload:
         """Generate a patch payload for updating a task.
 
-        Parameters
-        ----------
-        existing : BaseTask
-            The existing Task object.
-        updated : BaseTask
-            The updated Task object.
+         Parameters
+         ----------
+         existing : BaseTask
+             The existing Task object.
+         updated : BaseTask
+             The updated Task object.
 
-        Returns
-        -------
-        tuple[dict, dict[str, list[str]]]
-            The patch payload for updating the task and metadata list updates.
+         Returns
+         -------
+        TaskPatchPayload
+             The patch payload for updating the task
         """
-        _updatable_attributes_special = {"inventory_information"}
-        base_payload, list_metadata_updates = self._generate_patch_payload(
+        _updatable_attributes_special = {
+            "inventory_information",
+            "assigned_to",
+        }
+        if updated.assigned_to is not None:
+            updated.assigned_to = EntityLinkWithName(
+                id=updated.assigned_to.id, name=updated.assigned_to.name
+            )
+        base_payload = self._generate_task_patch_payload(
             existing=existing,
             updated=updated,
         )
-        patch_payload = base_payload.model_dump(mode="json", by_alias=True)
 
         for attribute in _updatable_attributes_special:
             old_value = getattr(existing, attribute)
             new_value = getattr(updated, attribute)
+
+            if attribute == "assigned_to":
+                if new_value == old_value or (
+                    new_value and old_value and new_value.id == old_value.id
+                ):
+                    continue
+                if old_value is None:
+                    base_payload.data.append(
+                        {
+                            "operation": PatchOperation.ADD,
+                            "attribute": "AssignedTo",
+                            "newValue": new_value,
+                        }
+                    )
+                    continue
+
+                if new_value is None:
+                    base_payload.data.append(
+                        {
+                            "operation": PatchOperation.DELETE,
+                            "attribute": "AssignedTo",
+                            "oldValue": old_value,
+                        }
+                    )
+                    continue
+                base_payload.data.append(
+                    {
+                        "operation": PatchOperation.UPDATE,
+                        "attribute": "AssignedTo",
+                        "oldValue": EntityLink(
+                            id=old_value.id
+                        ),  # can't include name with the old value or you get an error
+                        "newValue": new_value,
+                    }
+                )
+
             if attribute == "inventory_information":
-                existing_unique = [f"{x.inventory_id}#{x.lot_id}" for x in old_value]
-                updated_unique = [f"{x.inventory_id}#{x.lot_id}" for x in new_value]
-                inv_to_remove = []
-                for i, inv in enumerate(existing_unique):
-                    if inv not in updated_unique:
-                        inv_to_remove.append(
-                            old_value[i].model_dump(mode="json", by_alias=True, exclude_none=True)
-                        )
-                if len(inv_to_remove) > 0:
-                    patch_payload["data"].append(
+                existing_unique = {f"{x.inventory_id}#{x.lot_id}": x for x in old_value}
+                updated_unique = {f"{x.inventory_id}#{x.lot_id}": x for x in new_value}
+
+                # Find items to remove (in existing but not in updated)
+                inv_to_remove = [
+                    item.model_dump(mode="json", by_alias=True, exclude_none=True)
+                    for key, item in existing_unique.items()
+                    if key not in updated_unique
+                ]
+
+                # Find items to add (in updated but not in existing)
+                inv_to_add = [
+                    item.model_dump(mode="json", by_alias=True, exclude_none=True)
+                    for key, item in updated_unique.items()
+                    if key not in existing_unique
+                ]
+
+                if inv_to_remove:
+                    base_payload.data.append(
                         {
                             "operation": PatchOperation.DELETE,
                             "attribute": "inventory",
                             "oldValue": inv_to_remove,
                         }
                     )
-                inv_to_add = []
-                for i, inv in enumerate(updated_unique):
-                    if inv not in existing_unique:
-                        inv_to_add.append(
-                            new_value[i].model_dump(mode="json", by_alias=True, exclude_none=True)
-                        )
-                if len(inv_to_add) > 0:
-                    patch_payload["data"].append(
+
+                if inv_to_add:
+                    base_payload.data.append(
                         {
                             "operation": PatchOperation.ADD,
                             "attribute": "inventory",
@@ -463,7 +583,7 @@ class TaskCollection(BaseCollection):
                         }
                     )
 
-        return patch_payload, list_metadata_updates
+        return base_payload
 
     def update(self, *, task: BaseTask) -> BaseTask:
         """Update a task.
@@ -479,51 +599,20 @@ class TaskCollection(BaseCollection):
             The updated Task object as it exists in the Albert platform.
         """
         existing = self.get_by_id(id=task.id)
-        patch_payload, list_metadata_updates = self._generate_adv_patch_payload(
-            updated=task, existing=existing
-        )
-        patch_operations = patch_payload.get("data", [])
+        patch_payload = self._generate_adv_patch_payload(updated=task, existing=existing)
 
-        if len(patch_operations) == 0 and len(list_metadata_updates) == 0:
+        if len(patch_payload.data) == 0:
             logger.info(f"Task {task.id} is already up to date")
             return task
         path = f"{self.base_path}/{task.id}"
 
-        for datum in patch_operations:
+        for datum in patch_payload.data:
             patch_payload = TaskPatchPayload(data=[datum], id=task.id)
             self.session.patch(
                 url=path,
                 json=[patch_payload.model_dump(mode="json", by_alias=True, exclude_none=True)],
             )
 
-        # For metadata list field updates, we clear, then update
-        # since duplicate attribute values are not allowed in single patch request.
-        for attribute, values in list_metadata_updates.items():
-            entity_links = existing.metadata.get(attribute.split(".")[1])
-            old_values = [item.id if hasattr(item, "id") else item for item in entity_links]
-            clear_datum = PatchDatum(
-                operation=PatchOperation.DELETE, attribute=attribute, oldValue=old_values
-            )
-            clear_payload = TaskPatchPayload(data=[clear_datum], id=task.id)
-            self.session.patch(
-                url=path,
-                json=[clear_payload.model_dump(mode="json", by_alias=True, exclude_none=True)],
-            )
-            if values:
-                update_datum = PatchDatum(
-                    operation=PatchOperation.UPDATE,
-                    attribute=attribute,
-                    newValue=values,
-                    oldValue=[],
-                )
-
-                update_payload = TaskPatchPayload(data=[update_datum], id=task.id)
-                self.session.patch(
-                    url=path,
-                    json=[
-                        update_payload.model_dump(mode="json", by_alias=True, exclude_none=False)
-                    ],
-                )
         return self.get_by_id(id=task.id)
 
     def get_history(
