@@ -5,7 +5,7 @@ from contextlib import suppress
 
 import pytest
 
-from albert import Albert, ClientCredentials
+from albert import Albert, AlbertClientCredentials
 from albert.collections.worksheets import WorksheetCollection
 from albert.exceptions import BadRequestError, ForbiddenError, NotFoundError
 from albert.resources.btdataset import BTDataset
@@ -24,6 +24,7 @@ from albert.resources.lots import Lot
 from albert.resources.parameter_groups import ParameterGroup
 from albert.resources.parameters import Parameter
 from albert.resources.projects import Project
+from albert.resources.reports import FullAnalyticalReport
 from albert.resources.roles import Role
 from albert.resources.sheets import Component, Sheet
 from albert.resources.tags import Tag
@@ -54,6 +55,7 @@ from tests.seeding import (
     generate_parameter_seeds,
     generate_pricing_seeds,
     generate_project_seeds,
+    generate_report_seeds,
     generate_storage_location_seeds,
     generate_tag_seeds,
     generate_task_seeds,
@@ -65,13 +67,14 @@ from tests.utils.fake_session import FakeAlbertSession
 
 @pytest.fixture(scope="session")
 def client() -> Albert:
-    credentials = ClientCredentials.from_env(
+    credentials = AlbertClientCredentials.from_env(
         client_id_env="ALBERT_CLIENT_ID_SDK",
         client_secret_env="ALBERT_CLIENT_SECRET_SDK",
+        base_url_env="ALBERT_BASE_URL",
     )
     return Albert(
         base_url="https://app.albertinvent.com",
-        client_credentials=credentials,
+        auth_manager=credentials,
         retries=3,
     )
 
@@ -136,12 +139,12 @@ def static_sds_file(client: Albert) -> FileInfo:
 @pytest.fixture(scope="session")
 def static_roles(client: Albert) -> list[Role]:
     # Roles are not deleted or created. We just use the existing roles.
-    return list(client.roles.list())
+    return list(client.roles.get_all())
 
 
 @pytest.fixture(scope="session")
 def static_consumeable_parameter(client: Albert) -> Parameter:
-    consumeables = client.parameters.list(names="Consumables")
+    consumeables = client.parameters.get_all(names="Consumables")
     for c in consumeables:
         if c.name == "Consumables":
             return c
@@ -186,14 +189,19 @@ def static_lists(
 
 
 @pytest.fixture(scope="session")
-def seeded_cas(client: Albert, seed_prefix: str) -> Iterator[list[Cas]]:
+def seeded_cas(
+    client: Albert,
+    seed_prefix: str,
+    static_custom_fields: list[CustomField],
+    static_lists: list[ListItem],
+) -> Iterator[list[Cas]]:
     seeded = []
-    for cas in generate_cas_seeds(seed_prefix):
-        created_cas = client.cas_numbers.create(cas=cas)
+    for cas in generate_cas_seeds(seed_prefix, static_custom_fields, static_lists):
+        created_cas = client.cas_numbers.get_or_create(cas=cas)
         seeded.append(created_cas)
 
     # Avoid race condition while it populated through DBs
-    time.sleep(1.5)
+    time.sleep(3)
 
     yield seeded
 
@@ -206,7 +214,7 @@ def seeded_cas(client: Albert, seed_prefix: str) -> Iterator[list[Cas]]:
 def seeded_locations(client: Albert, seed_prefix: str) -> Iterator[list[Location]]:
     seeded = []
     for location in generate_location_seeds(seed_prefix):
-        created_location = client.locations.create(location=location)
+        created_location = client.locations.get_or_create(location=location)
         seeded.append(created_location)
 
     yield seeded
@@ -240,7 +248,7 @@ def seeded_projects(
 def seeded_companies(client: Albert, seed_prefix: str) -> Iterator[list[Company]]:
     seeded = []
     for company in generate_company_seeds(seed_prefix):
-        created_company = client.companies.create(company=company)
+        created_company = client.companies.get_or_create(company=company)
         seeded.append(created_company)
 
     yield seeded
@@ -258,7 +266,9 @@ def seeded_storage_locations(
 ) -> Iterator[list[Location]]:
     seeded = []
     for storage_location in generate_storage_location_seeds(seeded_locations=seeded_locations):
-        created_location = client.storage_locations.create(storage_location=storage_location)
+        created_location = client.storage_locations.get_or_create(
+            storage_location=storage_location
+        )
         seeded.append(created_location)
 
     yield seeded
@@ -272,7 +282,7 @@ def seeded_storage_locations(
 def seeded_tags(client: Albert, seed_prefix: str) -> Iterator[list[Tag]]:
     seeded = []
     for tag in generate_tag_seeds(seed_prefix):
-        created_tag = client.tags.create(tag=tag)
+        created_tag = client.tags.get_or_create(tag=tag)
         seeded.append(created_tag)
 
     yield seeded
@@ -286,7 +296,7 @@ def seeded_tags(client: Albert, seed_prefix: str) -> Iterator[list[Tag]]:
 def seeded_units(client: Albert, seed_prefix: str) -> Iterator[list[Unit]]:
     seeded = []
     for unit in generate_unit_seeds(seed_prefix):
-        created_unit = client.units.create(unit=unit)
+        created_unit = client.units.get_or_create(unit=unit)
         seeded.append(created_unit)
 
     # Avoid race condition while it populated through search DBs
@@ -332,6 +342,8 @@ def seeded_data_templates(
     static_user: User,
     seeded_data_columns: list[DataColumn],
     seeded_units: list[Unit],
+    seeded_tags: list[Tag],
+    seeded_parameters: list[Parameter],
 ) -> Iterator[list[DataTemplate]]:
     seeded = []
     for data_template in generate_data_template_seeds(
@@ -339,6 +351,8 @@ def seeded_data_templates(
         seed_prefix=seed_prefix,
         seeded_data_columns=seeded_data_columns,
         seeded_units=seeded_units,
+        seeded_tags=seeded_tags,
+        seeded_parameters=seeded_parameters,
     ):
         dt = client.data_templates.create(data_template=data_template)
         seeded.append(dt)
@@ -408,7 +422,7 @@ def seeded_inventory(
 def seeded_parameters(client: Albert, seed_prefix: str) -> Iterator[list[Parameter]]:
     seeded = []
     for parameter in generate_parameter_seeds(seed_prefix):
-        created_parameter = client.parameters.create(parameter=parameter)
+        created_parameter = client.parameters.get_or_create(parameter=parameter)
         # Extra get_by_id is required to populate the category field on parameter
         seeded.append(client.parameters.get_by_id(id=created_parameter.id))
     time.sleep(1.5)
@@ -547,7 +561,7 @@ def seeded_products(
         )
     return [
         x
-        for x in client.inventory.list(
+        for x in client.inventory.get_all(
             category=InventoryCategory.FORMULAS,
             text=product_name_prefix,
         )
@@ -624,8 +638,7 @@ def seeded_btdataset(client: Albert, seed_prefix: str) -> Iterator[BTDataset]:
     dataset = generate_btdataset_seed(seed_prefix)
     dataset = client.btdatasets.create(dataset=dataset)
     yield dataset
-    with suppress(ForbiddenError):  # TODO: Remove once ACL is fixed
-        client.btdatasets.delete(id=dataset.id)
+    client.btdatasets.delete(id=dataset.id)
 
 
 @pytest.fixture(scope="session")
@@ -637,8 +650,7 @@ def seeded_btmodelsession(
     model_session = generate_btmodelsession_seed(seed_prefix, seeded_btdataset)
     model_session = client.btmodelsessions.create(model_session=model_session)
     yield model_session
-    with suppress(ForbiddenError):  # TODO: Remove once ACL is fixed
-        client.btmodelsessions.delete(id=model_session.id)
+    client.btmodelsessions.delete(id=model_session.id)
 
 
 @pytest.fixture(scope="session")
@@ -649,10 +661,9 @@ def seeded_btmodel(
     seeded_btmodelsession: BTModelSession,
 ) -> Iterator[BTModel]:
     model = generate_btmodel_seed(seed_prefix, seeded_btdataset)
-    model = client.btmodels(parent_id=seeded_btmodelsession.id).create(model=model)
+    model = client.btmodels.create(model=model, parent_id=seeded_btmodelsession.id)
     yield model
-    with suppress(ForbiddenError):  # TODO: Remove once ACL is fixed
-        client.btmodels(parent_id=seeded_btmodelsession.id).delete(id=model.id)
+    client.btmodels.delete(id=model.id, parent_id=seeded_btmodelsession.id)
 
 
 @pytest.fixture(scope="session")
@@ -660,11 +671,29 @@ def seeded_btinsight(
     client: Albert,
     seed_prefix: str,
     seeded_btdataset: BTDataset,
-    seeded_btmodelsession: BTModelSession,
+    seeded_btmodelsession: BTModel,
 ) -> Iterator[BTInsight]:
     ins = generate_btinsight_seed(seed_prefix, seeded_btdataset, seeded_btmodelsession)
     ins = client.btinsights.create(insight=ins)
     time.sleep(3.0)
     yield ins
-    with suppress(ForbiddenError):  # TODO: Remove once ACL is fixed
-        client.btinsights.delete(id=ins.id)
+    client.btinsights.delete(id=ins.id)
+
+
+@pytest.fixture(scope="session")
+def seeded_reports(
+    client: Albert,
+    seed_prefix: str,
+    seeded_projects: list[Project],
+) -> Iterator[list[FullAnalyticalReport]]:
+    """Create seeded reports for testing."""
+    seeded = []
+    for report in generate_report_seeds(seed_prefix=seed_prefix, seeded_projects=seeded_projects):
+        created_report = client.reports.create_report(report=report)
+        seeded.append(created_report)
+
+    yield seeded
+
+    for report in seeded:
+        with suppress(NotFoundError):
+            client.reports.delete(id=report.id)
